-- ==================================
-- Simple parallax scrolling effect.
-- Art & code by Rafael Navega, 2024.
--
-- License: Public Domain
-- ==================================

local layerImages
local pixelCanvas

local cursorX, cursorY = 0.0, 0.0
local cursorSpeedX, cursorSpeedY = 0.0, 0.0
local lowestFactor

-- Completely subjective decay factor.
local DECAY = 1.2

function love.load()
    love.window.setMode(640, 400)
    love.window.setTitle('Move the mouse around; Press Esc to quit')

    love.graphics.setDefaultFilter('nearest', 'nearest')

    layerImages = {
        {love.graphics.newImage('Layer 0.png'), factor = 0.6},
        {love.graphics.newImage('Layer 1.png'), factor = 1.0},
        {love.graphics.newImage('Layer 2.png'), factor = 3.0},
        {love.graphics.newImage('Layer 3.png'), factor = 6.0},
        {love.graphics.newImage('Layer 4.png'), factor = 10.0},
        {love.graphics.newImage('Layer 5.png'), factor = nil},
    }
    -- The first layer is the one that moves around the most.
    -- Mark its scrolling factor so we can check when we've reached the limit
    -- for scrolling (an 80px border on all layers, chosen while drawing them).
    lowestFactor = layerImages[1].factor
    pixelCanvas = love.graphics.newCanvas(320, 200)
    love.mouse.setPosition(320, 200)
end


function love.update(dt)
    cursorSpeedX = cursorSpeedX * (1.0 - DECAY * dt)
    cursorSpeedY = cursorSpeedY * (1.0 - DECAY * dt)
    if math.abs(cursorSpeedX) < 0.001 then
        cursorSpeedX = 0.0
    end
    if math.abs(cursorSpeedY) < 0.001 then
        cursorSpeedY = 0.0
    end
    cursorX = cursorX + cursorSpeedX
    cursorY = cursorY + cursorSpeedY
    local limitX = cursorX / lowestFactor
    if limitX > 80.0 or limitX < -80.0 then
        cursorX = 80.0 * lowestFactor * (cursorX > 0 and 1.0 or -1.0)
        cursorSpeedX = 0.0
    end
    local limitY = cursorY / lowestFactor
    if limitY > 80.0 or limitY < -80.0 then
        cursorY = 80.0 * lowestFactor * (cursorY > 0 and 1.0 or -1.0)
        cursorSpeedY = 0.0
    end
end


function love.keypressed(key)
    if key == 'escape' then
        love.event.quit()
    end
end


function love.mousemoved(x, y, dx, dy)
    cursorSpeedX = cursorSpeedX + dx * 0.01
    cursorSpeedY = cursorSpeedY + dy * 0.01
end


function love.draw()
    love.graphics.setCanvas(pixelCanvas)
    love.graphics.clear()
    love.graphics.translate(-80.0, -80.0)
    for index = #layerImages, 1, -1 do
        local parallaxFactor = layerImages[index].factor
        local drawX, drawY = 0.0, 0.0
        if parallaxFactor then
            local deltaX = cursorX / parallaxFactor
            local deltaY = cursorY / parallaxFactor
            drawX = drawX + deltaX
            drawY = drawY + deltaY
        end
        love.graphics.draw(layerImages[index][1], drawX, drawY)
    end

    love.graphics.origin()
    love.graphics.setCanvas()
    love.graphics.draw(pixelCanvas, 0, 0, 0, 2.0, 2.0)
    love.graphics.print(string.format('%.02f, %.02f', cursorSpeedX, cursorSpeedY), 10, 10)
end
